using System;
using System.Web.UI;

using CMS.CMSHelper;
using CMS.GlobalHelper;
using CMS.PortalEngine;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.Staging;
using CMS.TreeEngine;
using CMS.UIControls;
using CMS.WorkflowEngine;
using CMS.FormControls;

public partial class CMSDesk_Content_Properties_template : CMSDeskPage, ICallbackEventHandler
{
    #region "Variables"

    protected int nodeId = 0;

    protected string mSave = null;
    protected string mInherit = null;
    protected string mClone = null;
    protected string mEditTemplateProperties = null;
    protected string mSaveDoc = null;

    private string btnSaveOnClickScript = "";
    private int siteid = 0;
    private int cloneId = 0;

    protected string imagesUrl = null;
    protected bool hasModifyPermission = true;

    protected TreeNode node = null;
    protected CurrentUserInfo currentUser = null;

    CMSUserControl InheritLevels1 = null;

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        // Register the scripts
        ScriptHelper.RegisterProgress(this.Page);
        
        this.InheritLevels1 = this.Page.LoadControl("~/CMSFormControls/InheritLevels.ascx") as CMSUserControl;
        this.InheritLevels1.ID = "ctrlInheritesLevels";
        this.InheritLevels1.SetValue("DocumentSettings", true);
        this.plcInherits.Controls.Add(this.InheritLevels1);

        UIContext.PropertyTab = PropertyTabEnum.Template;

        currentUser = CMSContext.CurrentUser;

        imagesUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSDesk/Content/Properties/Template/");

        // Register the script to perform get flags for showing buttons retrieval callback (if template is selected from dialog)
        Page.ClientScript.RegisterClientScriptBlock(GetType(), "ShowButtonFlags", ScriptHelper.GetScript("function GetShowButtonFlags(templateId){ return " + Page.ClientScript.GetCallbackEventReference(this, "templateId", "ReceiveShowButtonFlags", null) + " } \n"));

        // Register the dialog script
        Page.ClientScript.RegisterClientScriptBlock(typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY, ScriptHelper.DialogScript);

        ltlElemScript.Text += ScriptHelper.GetScript(
            "var cloneElem = document.getElementById('" + btnClone.ClientID + "'); \n" +
                "if (cloneElem != null) var cloneElemStyle = (cloneElem.style != null) ? cloneElem.style : cloneElem; \n" +
            "var inheritElem = document.getElementById('" + btnInherit.ClientID + "'); \n" +
                "if (inheritElem != null) var inheritElemStyle = (inheritElem.style != null) ? inheritElem.style : inheritElem; \n" +
            "var saveElem = document.getElementById('" + btnSave.ClientID + "'); \n" +
                "if (saveElem != null) var saveElemStyle = (saveElem.style != null) ? saveElem.style : saveElem; \n" +
            "var editTemplatePropertiesElem = document.getElementById('" + btnEditTemplateProperties.ClientID + "'); \n" +
                "if (editTemplatePropertiesElem != null) var editTemplatePropertiesElemStyle = (editTemplatePropertiesElem.style != null) ? editTemplatePropertiesElem.style : editTemplatePropertiesElem; \n"
            );

        nodeId = QueryHelper.GetInteger("nodeid", 0);

        TreeProvider tree = new TreeProvider(currentUser);
        node = DocumentHelper.GetDocument(nodeId, CMSContext.PreferredCultureCode, tree);

        if (node != null)
        {
            siteid = node.NodeSiteID;
        }

        txtTemplate.Text = ValidationHelper.GetString(Request.Params["txtTemplate"], "");
        lblTemplate.Text = ResHelper.GetString("PageProperties.Template");
        lblLevels.Text = ResHelper.GetString("PageProperties.InheritLevels");

        mClone = ResHelper.GetString("PageProperties.Clone");
        mSave = ResHelper.GetString("PageProperties.Save");
        mInherit = ResHelper.GetString("PageProperties.Inherit");
        mEditTemplateProperties = ResHelper.GetString("PageProperties.EditTemplateProperties");

        imgClone.ImageUrl = imagesUrl + "clone.gif";
        imgInherit.ImageUrl = imagesUrl + "inherit.gif";
        imgSave.ImageUrl = imagesUrl + "save.gif";
        imgEditTemplateProperties.ImageUrl = imagesUrl + "edit.gif";

        imgSaveDoc.ImageUrl = UrlHelper.ImagesDirectory + "/CMSDesk/Content/save.gif";
        mSaveDoc = ResHelper.GetString("general.save");

        btnEditTemplateProperties.Attributes.Add("onclick", "modalDialog('" + ResolveUrl("~/CMSSiteManager/Development/PageTemplates/PageTemplate_Edit.aspx") + "?templateid=' + document.getElementById('SelectedTemplateId').value + '&nobreadcrumbs=1', 'Template selection', 700, 550);return false;");

        ltlScript.Text = "";

        // Define GetCurrentTemplateId() used for specifing teplateId in the SaveAsNewTemplate onClick handler
        string initScript = "";
        initScript += "function GetCurrentTemplateId() \n";
        initScript += "{ \n";
        initScript += "if (document.getElementById('SelectedTemplateId').value > 0) \n";
        initScript += "{ \n";
        initScript += "return document.getElementById('SelectedTemplateId').value; \n";
        initScript += "} \n";
        initScript += "else \n";
        initScript += "{ \n";
        initScript += "return document.getElementById('InheritedTemplateId').value; \n";
        initScript += "} \n";
        initScript += "} \n";
        ltlPreInitScript.Text = ScriptHelper.GetScript(initScript);

        if (!RequestHelper.IsPostBack())
        {
            // Functions that are visible for global administrator
            if (currentUser.IsGlobalAdministrator || currentUser.IsAuthorizedPerResource("CMS.Content", "Design"))
            {
                btnSave.Visible = true;
                btnClone.Visible = true;
                btnEditTemplateProperties.Visible = true;
            }
            else
            {
                // Functions that are visible for other users
                btnSave.Visible = false;
                btnClone.Visible = false;
                btnEditTemplateProperties.Visible = false;
            }
            ReloadData();

            btnSaveOnClickScript = "modalDialog('" + ResolveUrl("~/CMSDesk/PortalEngine/SaveNewPageTemplate.aspx") + "?templateId=' + GetCurrentTemplateId() + '&siteid=" + siteid + "', 'Save new template', 470, 360);return false;";

            if (node != null)
            {
                if (node.NodeAliasPath != "/")
                {
                    InheritLevels1.SetValue("SelectedLevels", node.NodeInheritPageLevels);
                    InheritLevels1.SetValue("TreePath", TreePathUtils.GetParentPath(node.DocumentNamePath));
                }
                else
                {
                    plcInherits.Visible = false;
                }
            }
        }
        else
        {
            initScript = "";
            initScript += "document.getElementById('SelectedTemplateId').value = " + ValidationHelper.GetInteger(Request.Params["SelectedTemplateId"], 0) + "; \n ";
            initScript += "document.getElementById('InheritedTemplateId').value = " + ValidationHelper.GetInteger(Request.Params["InheritedTemplateId"], 0) + "; \n ";
            initScript += "document.getElementById('Saved').value = " + ValidationHelper.GetBoolean(Request.Params["Saved"], false).ToString().ToLower() + "; \n";
            initScript += "document.getElementById('TemplateDisplayName').value = '" + ValidationHelper.GetString(Request.Params["TemplateDisplayName"], "") + "'; \n ";
            initScript += "document.getElementById('TemplateDescription').value = '" + ValidationHelper.GetString(Request.Params["TemplateDescription"], "") + "'; \n ";
            initScript += "document.getElementById('TemplateCategory').value = '" + ValidationHelper.GetString(Request.Params["TemplateCategory"], "") + "'; \n ";
            initScript += "document.getElementById('isPortal').value = " + ValidationHelper.GetBoolean(Request.Params["isPortal"], false).ToString().ToLower() + "; \n ";
            initScript += "document.getElementById('isReusable').value = " + ValidationHelper.GetBoolean(Request.Params["isReusable"], false).ToString().ToLower() + "; \n ";
            initScript += "document.getElementById('isAdHoc').value = " + ValidationHelper.GetBoolean(Request.Params["isAdHoc"], false).ToString().ToLower() + "; \n ";

            string textTemplate = ValidationHelper.GetString(Request.Params["txtTemplate"], "");
            if (textTemplate == "")
            {
                textTemplate = ValidationHelper.GetString(Request.Params["TextTemplate"], "");
            }
            initScript += "document.getElementById('TextTemplate').value = " + ScriptHelper.GetString(textTemplate) + "; \n ";

            ltlInitScript.Text = ScriptHelper.GetScript(initScript);
            ltlScript.Text += ScriptHelper.GetScript("ShowButtons(document.getElementById('isPortal').value, document.getElementById('isReusable').value, document.getElementById('isAdHoc').value); \n");

            ltlScript.Text += ScriptHelper.GetScript("if (document.getElementById('SelectedTemplateId').value == 0) { if (inheritElemStyle != null) inheritElemStyle.display = 'none'; if (editTemplatePropertiesElemStyle != null) editTemplatePropertiesElemStyle.display = 'none'; }");
            txtTemplate.Text = textTemplate;

            btnSaveOnClickScript = "modalDialog('" + ResolveUrl("~/CMSDesk/PortalEngine/SaveNewPageTemplate.aspx") + "?templateId=' + GetCurrentTemplateId() + '&siteid=" + siteid + "', 'Save new template', 470, 360);return false;";
        }

        btnSelect.Text = ResHelper.GetString("PageProperties.Select");
        btnSelect.Attributes.Add("onclick", "modalDialog('" + ResolveUrl("~/CMSDesk/PortalEngine/SelectTemplate.aspx") + "?selectedPageTemplateId='+document.getElementById('SelectedTemplateId').value, 'Page template selection', 850, 520);return false;");
    }


    protected void Page_PreRender(object sender, EventArgs e)
    {
        btnSave.Attributes.Add("onclick", btnSaveOnClickScript);

        if (hasModifyPermission)
        {
            Page.ClientScript.RegisterClientScriptBlock(typeof(string), "EditShortcuts",
                    "<script type=\"text/javascript\" src=\"" + ResolveUrl("~/CMSScripts/shortcuts.js") + "\"></script>" +
                    ScriptHelper.GetScript("function SaveDocument() { " + ClientScript.GetPostBackEventReference(lnkSaveDoc, null) + " } \n"
                ));
        }
    }

    #endregion


    #region "Private methods"

    /// <summary>
    /// Disables form editing.
    /// </summary>
    private void DisableFormEditing()
    {
        hasModifyPermission = false;

        //pnlSoftTemplate.Enabled = false;
        lblTemplate.Enabled = false;
        txtTemplate.Enabled = false;
        btnSelect.Enabled = false;
        btnInherit.Enabled = false;
        btnClone.Enabled = false;
        plcInherits.Visible = false;
        btnSave.Enabled = true;
        btnEditTemplateProperties.Enabled = true;

        // Disable 'save button'
        lnkSaveDoc.Enabled = false;
        lnkSaveDoc.CssClass = "MenuItemEditDisabled";
        imgSaveDoc.ImageUrl = UrlHelper.ImagesDirectory + "/CMSDesk/Content/savedisabled.gif";
    }


    private void InheritTemplate(bool logSynchronization)
    {
        TreeProvider tree = new TreeProvider(currentUser);
        TreeNode node = DocumentHelper.GetDocument(nodeId, currentUser.PreferredCultureCode, tree);

        int pageTemplateId = 0;
        if ((node != null) && (node.NodeParentID > 0))
        {
            TreeNode parent = DocumentHelper.GetDocument(node.NodeParentID, CMSContext.PreferredCultureCode, tree);
            if (parent != null)
            {
                while (parent != null)
                {
                    if (parent.DocumentPageTemplateID > 0)
                    {
                        pageTemplateId = parent.DocumentPageTemplateID;
                        break;
                    }
                    parent = DocumentHelper.GetDocument(parent.NodeParentID, CMSContext.PreferredCultureCode, tree);
                }
            }
        }

        if (pageTemplateId > 0)
        {
            PageTemplateInfo pt = PageTemplateInfoProvider.GetPageTemplateInfo(pageTemplateId);
            if (pt != null)
            {
                txtTemplate.Text = pt.DisplayName + " (inherited)";
                ltlScript.Text += ScriptHelper.GetScript("pressedInherit(" + pageTemplateId + "); ShowButtons(" + pt.IsPortal.ToString().ToLower() + ", " + pt.IsReusable.ToString().ToLower() + ", false);");
                btnSaveOnClickScript = "modalDialog('" + ResolveUrl("~/CMSDesk/PortalEngine/SaveNewPageTemplate.aspx") + "?templateid=' + document.getElementById('InheritedTemplateId').value + '&siteid=" + siteid + "', 'Save new template', 470, 360);return false;";
            }
        }
        else
        {
            txtTemplate.Text = "";
            ltlScript.Text += ScriptHelper.GetScript("pressedInherit(" + pageTemplateId + "); ShowButtons(false, false, false); ");
        }

        if (logSynchronization)
        {
            // Log the synchronization
            DocumentHelper.LogSynchronization(node, TaskTypeEnum.UpdateDocument, tree);
        }
    }


    private void ReloadData()
    {
        TreeProvider tree = new TreeProvider(currentUser);

        node = tree.SelectSingleNode(nodeId, CMSContext.PreferredCultureCode);

        if (node != null)
        {
            // Check read permissions
            if (currentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
            {
                RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), node.NodeAliasPath));
            }
            // Check modify permissions
            else if (currentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
            {
                DisableFormEditing();

                lblInfo.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);
            }


            // Show tmplate name by template id
            if (node.GetValue("DocumentPageTemplateID") != null)
            {
                PageTemplateInfo pt = PageTemplateInfoProvider.GetPageTemplateInfo(Convert.ToInt32(node.GetValue("DocumentPageTemplateID")));
                if (pt != null)
                {
                    bool isAdHoc = false;
                    if ((pt.DisplayName.Length > 5) && (pt.DisplayName.Substring(0, 6) == "ad-hoc"))
                    {
                        isAdHoc = true;
                    }
                    ltlScript.Text += ScriptHelper.GetScript("SelectTemplate(" + pt.PageTemplateId + ", " + ScriptHelper.GetString(pt.DisplayName) + "); ShowButtons(" + pt.IsPortal.ToString().ToLower() + ", " + pt.IsReusable.ToString().ToLower() + ", " + isAdHoc.ToString().ToLower() + "); RememberTemplate(" + ScriptHelper.GetString(pt.DisplayName) + ");");
                }
                else
                {
                    ltlScript.Text += ScriptHelper.GetScript("NoTemplateSelected();");
                }
                btnInherit.Visible = true;
            }
            else
            {
                InheritTemplate(false);
            }
        }
    }

    #endregion


    #region "Button handling"

    protected void btnInherit_Click(object sender, EventArgs e)
    {
        InheritTemplate(true);
    }


    protected void btnClone_Click(object sender, EventArgs e)
    {
        int pageTemplateId = ValidationHelper.GetInteger(Request.Params["SelectedTemplateId"], 0);
        if (pageTemplateId == 0)
        {
            pageTemplateId = ValidationHelper.GetInteger(Request.Params["InheritedTemplateId"], 0);
        }

        if (pageTemplateId > 0)
        {
            PageTemplateInfo pt = PageTemplateInfoProvider.GetPageTemplateInfo(pageTemplateId);
            if (pt != null)
            {
                // Clone the info
                string docName = node.DocumentName;
                if (docName == "")
                {
                    docName = "/";
                }

                string displayName = "Ad-hoc: " + docName;

                PageTemplateInfo newInfo = PageTemplateInfoProvider.CloneTemplateAsAdHoc(pt, displayName, CMSContext.CurrentSite.SiteID);

                newInfo.Description = String.Format(ResHelper.GetString("PageTemplate.AdHocDescription"), node.DocumentNamePath);
                PageTemplateInfoProvider.SetPageTemplateInfo(newInfo);

                ltlScript.Text += ScriptHelper.GetScript("pressedClone(" + newInfo.PageTemplateId + "); ShowButtons(" + pt.IsPortal.ToString().ToLower() + ", " + pt.IsReusable.ToString().ToLower() + ", true);");
                btnSaveOnClickScript = "modalDialog('" + ResolveUrl("~/CMSDesk/PortalEngine/SaveNewPageTemplate.aspx") + "?templateid=' + document.getElementById('SelectedTemplateId').value + '&siteid=" + siteid + "', 'Save new template', 470, 360);return false;";
                txtTemplate.Text = newInfo.DisplayName;

                cloneId = newInfo.PageTemplateId;
            }
            btnSave.Visible = true;

            lnkSave_Click(sender, e);
        }
    }


    protected void lnkSave_Click(object sender, EventArgs e)
    {
        TreeProvider tree = new TreeProvider(currentUser);

        TreeNode node = tree.SelectSingleNode(nodeId, CMSContext.PreferredCultureCode);

        if (node != null)
        {
            // Update the data

            int selectedPageTemplateId = ValidationHelper.GetInteger(Request.Params["SelectedTemplateId"], 0);

            // Save just created ad-hoc template
            if (cloneId > 0)
            {
                selectedPageTemplateId = cloneId;
            }

            if (selectedPageTemplateId != 0)
            {
                ltlScript.Text += ScriptHelper.GetScript("document.getElementById('SelectedTemplateId').value = " + selectedPageTemplateId);
                if ((txtTemplate.Text.Length > 5) && (txtTemplate.Text.ToLower().Substring(0, 6) == "ad-hoc"))
                {
                    // Ad-hoc page is used, set ad-hoc page as DocumentPageTemplateID
                    node.SetValue("DocumentPageTemplateID", selectedPageTemplateId);

                    PageTemplateInfo pt = PageTemplateInfoProvider.GetPageTemplateInfo(selectedPageTemplateId);
                    if (pt != null)
                    {
                        ltlScript.Text += ScriptHelper.GetScript("pressedClone(" + selectedPageTemplateId + ",null); ShowButtons(" + pt.IsPortal.ToString().ToLower() + ", " + pt.IsReusable.ToString().ToLower() + ", true)");
                    }
                }
                else
                {
                    // Template was selected, set SelectedTemplateId as DocumentPageTemplateID
                    PageTemplateInfo pt = PageTemplateInfoProvider.GetPageTemplateInfo(selectedPageTemplateId);
                    if (pt != null)
                    {
                        node.SetValue("DocumentPageTemplateID", ValidationHelper.GetInteger(Request.Params["SelectedTemplateId"], 0));
                        ltlScript.Text += ScriptHelper.GetScript("SelectTemplate(" + selectedPageTemplateId + ",null); ShowButtons(" + pt.IsPortal.ToString().ToLower() + ", " + pt.IsReusable.ToString().ToLower() + ", false)");
                    }
                }
            }
            else
            {
                // Template is inherited, set null as DocumentPageTemplateID
                node.SetValue("DocumentPageTemplateID", null);
                int inheritedTemplateID = ValidationHelper.GetInteger(Request.Params["InheritedTemplateId"], 0);
                if (inheritedTemplateID > 0)
                {
                    PageTemplateInfo pt = PageTemplateInfoProvider.GetPageTemplateInfo(inheritedTemplateID);
                    if (pt != null)
                    {
                        ltlScript.Text += ScriptHelper.GetScript("pressedInherit(" + inheritedTemplateID + "); ShowButtons(" + pt.IsPortal.ToString().ToLower() + ", " + pt.IsReusable.ToString().ToLower() + ", false)");
                    }
                }
                else
                {
                    node.SetValue("DocumentPageTemplateID", null);
                    ltlScript.Text += ScriptHelper.GetScript("pressedInherit(0); ShowButtons(false, false, false); ");
                }
            }

            node.SetValue("NodeInheritPageLevels", InheritLevels1.GetValue("ExternalSelectedLevels"));
            node.Update();

            // Update search index for node
            if ((node.IsInPublishStep) && (SearchIndexInfoProvider.SearchEnabled))
            {
                SearchTaskInfoProvider.CreateTask(SearchTaskTypeEnum.Update, PredefinedObjectType.DOCUMENT, SearchHelper.ID_FIELD, node.GetSearchID());
            }

            lblInfo.Visible = true;
            lblInfo.Text = ResHelper.GetString("General.ChangesSaved");
        }

        // Log the synchronization
        DocumentHelper.LogSynchronization(node, TaskTypeEnum.UpdateDocument, tree);
    }

    #endregion


    #region "Callback handling"

    /// <summary>
    /// Current template description
    /// </summary>
    private int cbResultFlags = -1; // -1 = (false,...,false), 0 = (true,false,false), 1 = (true,true,false)
    //isAdhoc: 2 = (false,...,true),  3 = (true,false,true),  4 = (true,true,true)


    /// <summary>
    /// Callback event handler
    /// </summary>
    /// <param name="argument">Callback argument</param>
    public void RaiseCallbackEvent(string argument)
    {
        int pageTemplateId = Convert.ToInt32(argument);
        if (pageTemplateId > 0)
        {
            PageTemplateInfo pt = PageTemplateInfoProvider.GetPageTemplateInfo(pageTemplateId);
            if (pt != null)
            {
                if ((pt.IsPortal) && (pt.IsReusable))
                {
                    cbResultFlags = 1;
                }
                else if ((pt.IsPortal) && (!pt.IsReusable))
                {
                    cbResultFlags = 0;
                }
                else
                {
                    cbResultFlags = -1;
                }
                if ((pt.DisplayName.Length > 5) && (pt.DisplayName.Substring(0, 6) == "ad-hoc"))
                {
                    cbResultFlags += 3;
                }
            }
        }
    }


    /// <summary>
    /// Callback result retrieving handler
    /// </summary>
    public string GetCallbackResult()
    {
        return cbResultFlags.ToString();
    }

    #endregion
}

